extends Node2D
class_name Taquin
tool

var Piece = preload("res://src/Piece.tscn")

export var rows: int = 4
export var columns: int = 4
export var width: int = 512
export var height: int = 512

var pieces: Array = []
var missing_piece: Vector2

func _draw() -> void:
	draw_rect(Rect2(0, 0, width, height), Color.blue)
	
func _ready() -> void:
	for c in range(columns):
		var pieces_row: Array = []
		for r in range(rows):
			
			var piece = Piece.instance()

			var padding_w = (width - (piece.size * columns)) / (columns + 1)
			var padding_h = (height - (piece.size * rows)) / (rows + 1)
			var initial_position = Vector2(padding_w + c * (piece.size + padding_w), padding_h + r * (piece.size + padding_h))
			piece.set_number(1 + c + r * columns)
			piece.translate(initial_position)
			
			if r == rows - 1 && c == columns - 1:
				piece.visible = false
				missing_piece.x = c
				missing_piece.y = r

			add_child(piece)
			pieces_row.append(piece)

		pieces.append(pieces_row)

func _input(event):
	if event.is_action_pressed("ui_up"):
		move_piece(Direction.UP)
	if event.is_action_pressed("ui_down"):
		move_piece(Direction.DOWN)
	if event.is_action_pressed("ui_left"):
		move_piece(Direction.LEFT)
	if event.is_action_pressed("ui_right"):
		move_piece(Direction.RIGHT)

enum Direction { UP, DOWN, LEFT, RIGHT }

func move_piece(direction) -> bool:
	var destination: Vector2 = missing_piece
	match direction:
		Direction.UP:
			print("up")
			destination.y -= 1
		Direction.DOWN:
			destination.y += 1
			print("down")
		Direction.LEFT:
			destination.x -= 1
			print("left")
		Direction.RIGHT:
			destination.x += 1
			print("right")
	
	print(destination)
	if (destination.x < 0 || destination.x >= columns
	|| destination.y < 0 || destination.y >= rows):
		print("impossible move")
		return false

	swap(missing_piece, destination)
	missing_piece = destination
	update()
	return true

func swap(src: Vector2, dst: Vector2) -> void:
	var tmp: Piece = pieces[src.x][src.y]
	pieces[src.x][src.y] = pieces[dst.x][dst.y]
	pieces[dst.x][dst.y] = tmp
	pieces[src.x][src.y].position = position_for_index(src, tmp.size)
	pieces[dst.x][dst.y].position = position_for_index(dst, tmp.size)